import { EventEmitter2 } from '@nestjs/event-emitter';
import { AggregateRoot, Mapper, RepositoryPort } from '../ddd';
import { ObjectLiteral } from '../types';
import { LoggerPort } from '../ports/logger.port';
import {
  PrismaRawRepositoryPort,
  PrismaRepositoryPort,
} from '../ports/prisma-repository.port';
import { Prisma } from '@prisma/client';
import {
  ConflictException,
  DatabaseErrorException,
  NotFoundException,
} from '@libs/exceptions';

export abstract class PrismaRepositoryBase<
  Aggregate extends AggregateRoot<any>,
  DbReadModel extends ObjectLiteral,
  DbWriteModel extends ObjectLiteral,
> implements RepositoryPort<Aggregate>
{
  protected constructor(
    protected readonly prisma: PrismaRepositoryPort<Aggregate> | any,
    protected readonly prismaRaw: PrismaRawRepositoryPort,
    protected readonly mapper: Mapper<Aggregate, DbReadModel, DbWriteModel>,
    protected readonly eventEmitter: EventEmitter2,
    protected readonly logger: LoggerPort,
  ) {}

  async findOneById(id: string, include?: any): Promise<Aggregate> {
    const entity = await this.prisma.findUnique({
      where: { uuid: id },
      include,
    });
    if (entity) return this.mapper.toDomain(entity);
    throw new NotFoundException('Record not found');
  }

  async insert(entity: Aggregate): Promise<void> {
    try {
      await this.prisma.create({
        data: this.mapper.toPersistence(entity),
      });
    } catch (e) {
      if (e instanceof Prisma.PrismaClientKnownRequestError) {
        if (e.message.includes('Already exists')) {
          throw new ConflictException('Record already exists', e);
        }
      }
      throw e;
    }
  }

  async healthCheck(): Promise<boolean> {
    try {
      await this.prismaRaw.$queryRaw`SELECT 1`;
      return true;
    } catch (e) {
      if (e instanceof Prisma.PrismaClientKnownRequestError) {
        throw new DatabaseErrorException(e.message);
      }
      throw new DatabaseErrorException();
    }
  }
}
