import { Mapper } from '@libs/ddd';
import { AdResponseDto } from './interface/dtos/ad.response.dto';
import { Inject, Injectable } from '@nestjs/common';
import { AdEntity } from './core/ad.entity';
import {
  AdWriteModel,
  AdReadModel,
  WaypointModel,
} from './infrastructure/ad.repository';
import { Frequency } from './core/ad.types';
import { WaypointProps } from './core/value-objects/waypoint.value-object';
import { v4 } from 'uuid';
import {
  PARAMS_PROVIDER,
  TIMEZONE_FINDER,
  TIME_CONVERTER,
} from './ad.di-tokens';
import { TimezoneFinderPort } from './core/ports/timezone-finder.port';
import { DefaultParamsProviderPort } from './core/ports/default-params-provider.port';
import { DefaultParams } from './core/ports/default-params.type';
import { TimeConverterPort } from './core/ports/time-converter.port';

/**
 * Mapper constructs objects that are used in different layers:
 * Record is an object that is stored in a database,
 * Entity is an object that is used in application domain layer,
 * and a ResponseDTO is an object returned to a user (usually as json).
 */

@Injectable()
export class AdMapper
  implements Mapper<AdEntity, AdReadModel, AdWriteModel, AdResponseDto>
{
  private readonly defaultParams: DefaultParams;

  constructor(
    @Inject(PARAMS_PROVIDER)
    private readonly defaultParamsProvider: DefaultParamsProviderPort,
    @Inject(TIMEZONE_FINDER)
    private readonly timezoneFinder: TimezoneFinderPort,
    @Inject(TIME_CONVERTER)
    private readonly timeConverter: TimeConverterPort,
  ) {
    this.defaultParams = defaultParamsProvider.getParams();
  }

  toPersistence = (entity: AdEntity): AdWriteModel => {
    const copy = entity.getProps();
    const { lon, lat } = copy.waypoints[0].address.coordinates;
    const timezone = this.timezoneFinder.timezones(
      lon,
      lat,
      this.defaultParams.DEFAULT_TIMEZONE,
    )[0];
    const now = new Date();
    const record: AdWriteModel = {
      uuid: copy.id,
      userUuid: copy.userId,
      driver: copy.driver,
      passenger: copy.passenger,
      frequency: copy.frequency,
      fromDate: new Date(copy.fromDate),
      toDate: new Date(copy.toDate),
      monTime: copy.schedule.mon
        ? this.timeConverter.localDateTimeToUtc(
            copy.fromDate,
            copy.schedule.mon,
            timezone,
          )
        : undefined,
      tueTime: copy.schedule.tue
        ? this.timeConverter.localDateTimeToUtc(
            copy.fromDate,
            copy.schedule.tue,
            timezone,
          )
        : undefined,
      wedTime: copy.schedule.wed
        ? this.timeConverter.localDateTimeToUtc(
            copy.fromDate,
            copy.schedule.wed,
            timezone,
          )
        : undefined,
      thuTime: copy.schedule.thu
        ? this.timeConverter.localDateTimeToUtc(
            copy.fromDate,
            copy.schedule.thu,
            timezone,
          )
        : undefined,
      friTime: copy.schedule.fri
        ? this.timeConverter.localDateTimeToUtc(
            copy.fromDate,
            copy.schedule.fri,
            timezone,
          )
        : undefined,
      satTime: copy.schedule.sat
        ? this.timeConverter.localDateTimeToUtc(
            copy.fromDate,
            copy.schedule.sat,
            timezone,
          )
        : undefined,
      sunTime: copy.schedule.sun
        ? this.timeConverter.localDateTimeToUtc(
            copy.fromDate,
            copy.schedule.sun,
            timezone,
          )
        : undefined,
      monMargin: copy.marginDurations.mon,
      tueMargin: copy.marginDurations.tue,
      wedMargin: copy.marginDurations.wed,
      thuMargin: copy.marginDurations.thu,
      friMargin: copy.marginDurations.fri,
      satMargin: copy.marginDurations.sat,
      sunMargin: copy.marginDurations.sun,
      seatsProposed: copy.seatsProposed,
      seatsRequested: copy.seatsRequested,
      strict: copy.strict,
      waypoints: {
        create: copy.waypoints.map((waypoint: WaypointProps) => ({
          uuid: v4(),
          position: waypoint.position,
          name: waypoint.address.name,
          houseNumber: waypoint.address.houseNumber,
          street: waypoint.address.street,
          locality: waypoint.address.locality,
          postalCode: waypoint.address.postalCode,
          country: waypoint.address.country,
          lon: waypoint.address.coordinates.lon,
          lat: waypoint.address.coordinates.lat,
          createdAt: now,
          updatedAt: now,
        })),
      },
      createdAt: copy.createdAt,
      updatedAt: copy.updatedAt,
    };
    return record;
  };

  toDomain = (record: AdReadModel): AdEntity => {
    const timezone = this.timezoneFinder.timezones(
      record.waypoints[0].lon,
      record.waypoints[0].lat,
      this.defaultParams.DEFAULT_TIMEZONE,
    )[0];
    const entity = new AdEntity({
      id: record.uuid,
      createdAt: new Date(record.createdAt),
      updatedAt: new Date(record.updatedAt),
      props: {
        userId: record.userUuid,
        driver: record.driver,
        passenger: record.passenger,
        frequency: Frequency[record.frequency],
        fromDate: record.fromDate.toISOString().split('T')[0],
        toDate: record.toDate.toISOString().split('T')[0],
        schedule: {
          mon: record.monTime?.toISOString(),
          tue: record.tueTime?.toISOString(),
          wed: record.wedTime
            ? this.timeConverter.utcDatetimeToLocalTime(
                record.wedTime.toISOString(),
                timezone,
              )
            : undefined,
          thu: record.thuTime
            ? this.timeConverter.utcDatetimeToLocalTime(
                record.thuTime.toISOString(),
                timezone,
              )
            : undefined,
          fri: record.friTime?.toISOString(),
          sat: record.satTime?.toISOString(),
          sun: record.sunTime?.toISOString(),
        },
        marginDurations: {
          mon: record.monMargin,
          tue: record.tueMargin,
          wed: record.wedMargin,
          thu: record.thuMargin,
          fri: record.friMargin,
          sat: record.satMargin,
          sun: record.sunMargin,
        },
        seatsProposed: record.seatsProposed,
        seatsRequested: record.seatsRequested,
        strict: record.strict,
        waypoints: record.waypoints.map((waypoint: WaypointModel) => ({
          position: waypoint.position,
          address: {
            name: waypoint.name,
            houseNumber: waypoint.houseNumber,
            street: waypoint.street,
            postalCode: waypoint.postalCode,
            locality: waypoint.locality,
            country: waypoint.country,
            coordinates: {
              lon: waypoint.lon,
              lat: waypoint.lat,
            },
          },
        })),
      },
    });
    return entity;
  };

  toResponse = (entity: AdEntity): AdResponseDto => {
    const props = entity.getProps();
    const response = new AdResponseDto(entity);
    response.userId = props.userId;
    response.driver = props.driver;
    response.passenger = props.passenger;
    response.frequency = props.frequency;
    response.fromDate = props.fromDate;
    response.toDate = props.toDate;
    response.schedule = { ...props.schedule };
    response.marginDurations = { ...props.marginDurations };
    response.seatsProposed = props.seatsProposed;
    response.seatsRequested = props.seatsRequested;
    response.waypoints = props.waypoints.map((waypoint: WaypointProps) => ({
      position: waypoint.position,
      name: waypoint.address.name,
      houseNumber: waypoint.address.houseNumber,
      street: waypoint.address.street,
      postalCode: waypoint.address.postalCode,
      locality: waypoint.address.locality,
      country: waypoint.address.country,
      lon: waypoint.address.coordinates.lon,
      lat: waypoint.address.coordinates.lat,
    }));
    return response;
  };

  /* ^ Data returned to the user is whitelisted to avoid leaks.
     If a new property is added, like password or a
     credit card number, it won't be returned
     unless you specifically allow this.
     (avoid blacklisting, which will return everything
      but blacklisted items, which can lead to a data leak).
  */
}
