import { PrismaService } from '@libs/db/prisma.service';
import {
  AD_REPOSITORY,
  PARAMS_PROVIDER,
  TIMEZONE_FINDER,
  TIME_CONVERTER,
} from '@modules/ad/ad.di-tokens';
import { AdMapper } from '@modules/ad/ad.mapper';
import { AdRepository } from '@modules/ad/infrastructure/ad.repository';
import { DefaultParamsProvider } from '@modules/ad/infrastructure/default-params-provider';
import { TimeConverter } from '@modules/ad/infrastructure/time-converter';
import { TimezoneFinder } from '@modules/ad/infrastructure/timezone-finder';
import { ConfigModule } from '@nestjs/config';
import { EventEmitterModule } from '@nestjs/event-emitter';
import { Test } from '@nestjs/testing';

describe('Ad Repository', () => {
  let prismaService: PrismaService;
  let adRepository: AdRepository;

  const executeInsertCommand = async (table: string, object: any) => {
    const command = `INSERT INTO ${table} ("${Object.keys(object).join(
      '","',
    )}") VALUES (${Object.values(object).join(',')})`;

    await prismaService.$executeRawUnsafe(command);
  };
  const getSeed = (index: number, uuid: string): string => {
    return `'${uuid.slice(0, -2)}${index.toString(16).padStart(2, '0')}'`;
  };

  const baseUuid = {
    uuid: 'be459a29-7a41-4c0b-b371-abe90bfb6f00',
  };
  const baseOriginWaypointUuid = {
    uuid: 'bad5e786-3b15-4e51-a8fc-926fa9327ff1',
  };
  const baseDestinationWaypointUuid = {
    uuid: '4d200eb6-7389-487f-a1ca-dbc0e40381c9',
  };
  const baseUserUuid = {
    userUuid: "'113e0000-0000-4000-a000-000000000000'",
  };
  const driverAd = {
    driver: 'true',
    passenger: 'false',
    seatsProposed: 3,
    seatsRequested: 0,
    strict: 'false',
  };
  // const passengerAd = {
  //   driver: 'false',
  //   passenger: 'true',
  //   seatsProposed: 0,
  //   seatsRequested: 1,
  //   strict: 'false',
  // };
  // const driverAndPassengerAd = {
  //   driver: 'true',
  //   passenger: 'true',
  //   seatsProposed: 3,
  //   seatsRequested: 1,
  //   strict: 'false',
  // };
  const punctualAd = {
    frequency: `'PUNCTUAL'`,
    fromDate: `'2023-01-01'`,
    toDate: `'2023-01-01'`,
    monTime: 'NULL',
    tueTime: 'NULL',
    wedTime: 'NULL',
    thuTime: 'NULL',
    friTime: 'NULL',
    satTime: 'NULL',
    sunTime: `'2023-01-01T07:00:00Z'`,
    monMargin: 900,
    tueMargin: 900,
    wedMargin: 900,
    thuMargin: 900,
    friMargin: 900,
    satMargin: 900,
    sunMargin: 900,
  };
  // const recurrentAd = {
  //   frequency: `'RECURRENT'`,
  //   fromDate: `'2023-01-01'`,
  //   toDate: `'2023-12-31'`,
  //   monTime: `'2023-01-01T07:15:00Z'`,
  //   tueTime: `'2023-01-01T07:15:00Z'`,
  //   wedTime: `'2023-01-01T07:05:00Z'`,
  //   thuTime: `'2023-01-01T07:15:00Z'`,
  //   friTime: `'2023-01-01T07:15:00Z'`,
  //   satTime: 'NULL',
  //   sunTime: 'NULL',
  //   monMargin: 900,
  //   tueMargin: 900,
  //   wedMargin: 900,
  //   thuMargin: 900,
  //   friMargin: 900,
  //   satMargin: 900,
  //   sunMargin: 900,
  // };
  const originWaypoint = {
    position: 0,
    lon: 43.7102,
    lat: 7.262,
    locality: "'Nice'",
    postalCode: "'06000'",
    country: "'France'",
  };
  const destinationWaypoint = {
    position: 1,
    lon: 43.2965,
    lat: 5.3698,
    locality: "'Marseille'",
    postalCode: "'13000'",
    country: "'France'",
  };

  const createPunctualDriverAds = async (nbToCreate = 10) => {
    const adToCreate = {
      ...baseUuid,
      ...baseUserUuid,
      ...driverAd,
      ...punctualAd,
    };
    for (let i = 0; i < nbToCreate; i++) {
      adToCreate.uuid = getSeed(i, baseUuid.uuid);
      await executeInsertCommand('ad', adToCreate);
      await executeInsertCommand('waypoint', {
        uuid: getSeed(i, baseOriginWaypointUuid.uuid),
        adUuid: adToCreate.uuid,
        ...originWaypoint,
      });
      await executeInsertCommand('waypoint', {
        uuid: getSeed(i, baseDestinationWaypointUuid.uuid),
        adUuid: adToCreate.uuid,
        ...destinationWaypoint,
      });
    }
  };

  // const createRecurrentDriverAds = async (nbToCreate = 10) => {
  //   const adToCreate = {
  //     ...baseUuid,
  //     ...baseUserUuid,
  //     ...driverAd,
  //     ...punctualAd,
  //   };
  //   for (let i = 0; i < nbToCreate; i++) {
  //     adToCreate.uuid = getSeed(i, baseUuid.uuid);
  //     await executeInsertCommand('ad', adToCreate);
  //     await executeInsertCommand('waypoint', {
  //       uuid: getSeed(i, baseOriginWaypointUuid.uuid),
  //       adUuid: adToCreate.uuid,
  //       ...originWaypoint,
  //     });
  //     await executeInsertCommand('waypoint', {
  //       uuid: getSeed(i, baseDestinationWaypointUuid.uuid),
  //       adUuid: adToCreate.uuid,
  //       ...destinationWaypoint,
  //     });
  //   }
  // };

  // const createPunctualPassengerAds = async (nbToCreate = 10) => {
  //   const adToCreate = {
  //     ...baseUuid,
  //     ...baseUserUuid,
  //     ...passengerAd,
  //     ...punctualAd,
  //   };
  //   for (let i = 0; i < nbToCreate; i++) {
  //     adToCreate.uuid = getSeed(i, baseUuid.uuid);
  //     await executeInsertCommand('ad', adToCreate);
  //     await executeInsertCommand('waypoint', {
  //       uuid: getSeed(i, baseOriginWaypointUuid.uuid),
  //       adUuid: adToCreate.uuid,
  //       ...originWaypoint,
  //     });
  //     await executeInsertCommand('waypoint', {
  //       uuid: getSeed(i, baseDestinationWaypointUuid.uuid),
  //       adUuid: adToCreate.uuid,
  //       ...destinationWaypoint,
  //     });
  //   }
  // };

  // const createRecurrentPassengerAds = async (nbToCreate = 10) => {
  //   const adToCreate = {
  //     ...baseUuid,
  //     ...baseUserUuid,
  //     ...passengerAd,
  //     ...recurrentAd,
  //   };
  //   for (let i = 0; i < nbToCreate; i++) {
  //     adToCreate.uuid = getSeed(i, baseUuid.uuid);
  //     await executeInsertCommand('ad', adToCreate);
  //     await executeInsertCommand('waypoint', {
  //       uuid: getSeed(i, baseOriginWaypointUuid.uuid),
  //       adUuid: adToCreate.uuid,
  //       ...originWaypoint,
  //     });
  //     await executeInsertCommand('waypoint', {
  //       uuid: getSeed(i, baseDestinationWaypointUuid.uuid),
  //       adUuid: adToCreate.uuid,
  //       ...destinationWaypoint,
  //     });
  //   }
  // };

  // const createPunctualDriverPassengerAds = async (nbToCreate = 10) => {
  //   const adToCreate = {
  //     ...baseUuid,
  //     ...baseUserUuid,
  //     ...driverAndPassengerAd,
  //     ...punctualAd,
  //   };
  //   for (let i = 0; i < nbToCreate; i++) {
  //     adToCreate.uuid = getSeed(i, baseUuid.uuid);
  //     await executeInsertCommand('ad', adToCreate);
  //     await executeInsertCommand('waypoint', {
  //       uuid: getSeed(i, baseOriginWaypointUuid.uuid),
  //       adUuid: adToCreate.uuid,
  //       ...originWaypoint,
  //     });
  //     await executeInsertCommand('waypoint', {
  //       uuid: getSeed(i, baseDestinationWaypointUuid.uuid),
  //       adUuid: adToCreate.uuid,
  //       ...destinationWaypoint,
  //     });
  //   }
  // };

  // const createRecurrentDriverPassengerAds = async (nbToCreate = 10) => {
  //   const adToCreate = {
  //     ...baseUuid,
  //     ...baseUserUuid,
  //     ...driverAndPassengerAd,
  //     ...recurrentAd,
  //   };
  //   for (let i = 0; i < nbToCreate; i++) {
  //     adToCreate.uuid = getSeed(i, baseUuid.uuid);
  //     await executeInsertCommand('ad', adToCreate);
  //     await executeInsertCommand('waypoint', {
  //       uuid: getSeed(i, baseOriginWaypointUuid.uuid),
  //       adUuid: adToCreate.uuid,
  //       ...originWaypoint,
  //     });
  //     await executeInsertCommand('waypoint', {
  //       uuid: getSeed(i, baseDestinationWaypointUuid.uuid),
  //       adUuid: adToCreate.uuid,
  //       ...destinationWaypoint,
  //     });
  //   }
  // };

  beforeAll(async () => {
    const module = await Test.createTestingModule({
      imports: [
        EventEmitterModule.forRoot(),
        ConfigModule.forRoot({ isGlobal: true }),
      ],
      providers: [
        PrismaService,
        AdMapper,
        {
          provide: AD_REPOSITORY,
          useClass: AdRepository,
        },
        {
          provide: PARAMS_PROVIDER,
          useClass: DefaultParamsProvider,
        },
        {
          provide: TIMEZONE_FINDER,
          useClass: TimezoneFinder,
        },
        {
          provide: TIME_CONVERTER,
          useClass: TimeConverter,
        },
      ],
    }).compile();
    prismaService = module.get<PrismaService>(PrismaService);
    adRepository = module.get<AdRepository>(AD_REPOSITORY);
  });
  afterAll(async () => {
    await prismaService.$disconnect();
  });

  beforeEach(async () => {
    await prismaService.ad.deleteMany();
  });
  // describe('findAll', () => {
  //   it('should return an empty data array', async () => {
  //     const res = await adRepository.findAll();
  //     expect(res).toEqual({
  //       data: [],
  //       total: 0,
  //     });
  //   });

  //   describe('drivers', () => {
  //     it('should return a data array with 8 punctual driver ads', async () => {
  //       await createPunctualDriverAds(8);
  //       const ads = await adRepository.findAll();
  //       expect(ads.data.length).toBe(8);
  //       expect(ads.total).toBe(8);
  //       expect(ads.data[0].driver).toBeTruthy();
  //       expect(ads.data[0].passenger).toBeFalsy();
  //     });

  //     it('should return a data array limited to 10 punctual driver ads', async () => {
  //       await createPunctualDriverAds(20);
  //       const ads = await adRepository.findAll();
  //       expect(ads.data.length).toBe(10);
  //       expect(ads.total).toBe(20);
  //       expect(ads.data[1].driver).toBeTruthy();
  //       expect(ads.data[1].passenger).toBeFalsy();
  //     });

  //     it('should return a data array with 8 recurrent driver ads', async () => {
  //       await createRecurrentDriverAds(8);
  //       const ads = await adRepository.findAll();
  //       expect(ads.data.length).toBe(8);
  //       expect(ads.total).toBe(8);
  //       expect(ads.data[2].driver).toBeTruthy();
  //       expect(ads.data[2].passenger).toBeFalsy();
  //     });

  //     it('should return a data array limited to 10 recurrent driver ads', async () => {
  //       await createRecurrentDriverAds(20);
  //       const ads = await adRepository.findAll();
  //       expect(ads.data.length).toBe(10);
  //       expect(ads.total).toBe(20);
  //       expect(ads.data[3].driver).toBeTruthy();
  //       expect(ads.data[3].passenger).toBeFalsy();
  //     });
  //   });

  //   describe('passengers', () => {
  //     it('should return a data array with 7 punctual passenger ads', async () => {
  //       await createPunctualPassengerAds(7);
  //       const ads = await adRepository.findAll();
  //       expect(ads.data.length).toBe(7);
  //       expect(ads.total).toBe(7);
  //       expect(ads.data[0].passenger).toBeTruthy();
  //       expect(ads.data[0].driver).toBeFalsy();
  //     });

  //     it('should return a data array limited to 10 punctual passenger ads', async () => {
  //       await createPunctualPassengerAds(15);
  //       const ads = await adRepository.findAll();
  //       expect(ads.data.length).toBe(10);
  //       expect(ads.total).toBe(15);
  //       expect(ads.data[1].passenger).toBeTruthy();
  //       expect(ads.data[1].driver).toBeFalsy();
  //     });

  //     it('should return a data array with 7 recurrent passenger ads', async () => {
  //       await createRecurrentPassengerAds(7);
  //       const ads = await adRepository.findAll();
  //       expect(ads.data.length).toBe(7);
  //       expect(ads.total).toBe(7);
  //       expect(ads.data[2].passenger).toBeTruthy();
  //       expect(ads.data[2].driver).toBeFalsy();
  //     });

  //     it('should return a data array limited to 10 recurrent passenger ads', async () => {
  //       await createRecurrentPassengerAds(15);
  //       const ads = await adRepository.findAll();
  //       expect(ads.data.length).toBe(10);
  //       expect(ads.total).toBe(15);
  //       expect(ads.data[3].passenger).toBeTruthy();
  //       expect(ads.data[3].driver).toBeFalsy();
  //     });
  //   });

  //   describe('drivers and passengers', () => {
  //     it('should return a data array with 6 punctual driver and passenger ads', async () => {
  //       await createPunctualDriverPassengerAds(6);
  //       const ads = await adRepository.findAll();
  //       expect(ads.data.length).toBe(6);
  //       expect(ads.total).toBe(6);
  //       expect(ads.data[0].passenger).toBeTruthy();
  //       expect(ads.data[0].driver).toBeTruthy();
  //     });

  //     it('should return a data array limited to 10 punctual driver and passenger ads', async () => {
  //       await createPunctualDriverPassengerAds(16);
  //       const ads = await adRepository.findAll();
  //       expect(ads.data.length).toBe(10);
  //       expect(ads.total).toBe(16);
  //       expect(ads.data[1].passenger).toBeTruthy();
  //       expect(ads.data[1].driver).toBeTruthy();
  //     });

  //     it('should return a data array with 6 recurrent driver and passenger ads', async () => {
  //       await createRecurrentDriverPassengerAds(6);
  //       const ads = await adRepository.findAll();
  //       expect(ads.data.length).toBe(6);
  //       expect(ads.total).toBe(6);
  //       expect(ads.data[2].passenger).toBeTruthy();
  //       expect(ads.data[2].driver).toBeTruthy();
  //     });

  //     it('should return a data array limited to 10 recurrent driver and passenger ads', async () => {
  //       await createRecurrentDriverPassengerAds(16);
  //       const ads = await adRepository.findAll();
  //       expect(ads.data.length).toBe(10);
  //       expect(ads.total).toBe(16);
  //       expect(ads.data[3].passenger).toBeTruthy();
  //       expect(ads.data[3].driver).toBeTruthy();
  //     });
  //   });
  // });
  describe('findOneById', () => {
    it('should return an ad', async () => {
      await createPunctualDriverAds(1);
      const result = await adRepository.findOneById(baseUuid.uuid, {
        waypoints: true,
      });

      expect(result.id).toBe(baseUuid.uuid);
    });

    // it('should return null', async () => {
    //   const ad = await adRepository.findOneById(
    //     '544572be-11fb-4244-8235-587221fc9104',
    //   );
    //   expect(ad).toBeNull();
    // });
  });
  // describe('create', () => {
  // it('should create a punctual ad', async () => {
  //   const beforeCount = await prismaService.ad.count();
  //   const adToCreate: AdDTO = new AdDTO();

  //   adToCreate.uuid = 'be459a29-7a41-4c0b-b371-abe90bfb6f00';
  //   adToCreate.userUuid = '4e52b54d-a729-4dbd-9283-f84a11bb2200';
  //   adToCreate.driver = true;
  //   adToCreate.passenger = false;
  //   adToCreate.frequency = Frequency.PUNCTUAL;
  //   adToCreate.fromDate = new Date('05-22-2023 09:36');
  //   adToCreate.toDate = new Date('05-22-2023 09:36');
  //   adToCreate.monTime = '09:36';
  //   adToCreate.monMargin = 900;
  //   adToCreate.tueMargin = 900;
  //   adToCreate.wedMargin = 900;
  //   adToCreate.thuMargin = 900;
  //   adToCreate.friMargin = 900;
  //   adToCreate.satMargin = 900;
  //   adToCreate.sunMargin = 900;
  //   adToCreate.seatsProposed = 3;
  //   adToCreate.seatsRequested = 0;
  //   adToCreate.strict = false;
  //   adToCreate.waypoints = {
  //     create: [originWaypoint as Waypoint, destinationWaypoint as Waypoint],
  //   };
  //   const ad = await adRepository.create(adToCreate);

  //   const afterCount = await prismaService.ad.count();

  //   expect(afterCount - beforeCount).toBe(1);
  //   expect(ad.uuid).toBe('be459a29-7a41-4c0b-b371-abe90bfb6f00');
  // });
  // it('should create a recurrent ad', async () => {
  //   const beforeCount = await prismaService.ad.count();
  //   const adToCreate: AdDTO = new AdDTO();

  //   adToCreate.uuid = '137a26fa-4b38-48ba-aecf-1a75f6b20f3d';
  //   adToCreate.userUuid = '4e52b54d-a729-4dbd-9283-f84a11bb2200';
  //   adToCreate.driver = true;
  //   adToCreate.passenger = false;
  //   adToCreate.frequency = Frequency.RECURRENT;
  //   adToCreate.fromDate = new Date('01-15-2023 ');
  //   adToCreate.toDate = new Date('10-31-2023');
  //   adToCreate.monTime = '07:30';
  //   adToCreate.friTime = '07:45';
  //   adToCreate.thuTime = '08:00';
  //   adToCreate.monMargin = 900;
  //   adToCreate.tueMargin = 900;
  //   adToCreate.wedMargin = 900;
  //   adToCreate.thuMargin = 900;
  //   adToCreate.friMargin = 900;
  //   adToCreate.satMargin = 900;
  //   adToCreate.sunMargin = 900;
  //   adToCreate.seatsProposed = 2;
  //   adToCreate.seatsRequested = 0;
  //   adToCreate.strict = false;
  //   adToCreate.waypoints = {
  //     create: [originWaypoint as Waypoint, destinationWaypoint as Waypoint],
  //   };
  //   const ad = await adRepository.create(adToCreate);

  //   const afterCount = await prismaService.ad.count();

  //   expect(afterCount - beforeCount).toBe(1);
  //   expect(ad.uuid).toBe('137a26fa-4b38-48ba-aecf-1a75f6b20f3d');
  // });
  // });
});
